/*-------------------------------------------------------------------------
   Register Declarations for the Dallas DS89C420 Processor
   
   based on: 
   
   DS89C420 Ultra High Speed Microcontroller User Guide
   -January 24, 2001 
   -020602
   
   DS89C420 Ultra High Speed Microcontroller
   -September 22, 2000 
   
   Written By -  Rainer Stamm / rainer.stamm@gmx.de 
   (Feb 2002)
    
   History:
   ------------------------------------------------------------------------
   Version 1.0      Feb 2002
                    first release
   ------------------------------------------------------------------------
   Version 1.0.1    March 8, 2002
                    Interrupt numbers corrected (PFI_VECTOR - WDI_VECTOR)
   ------------------------------------------------------------------------
   
   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2, or (at your option) any
   later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

   In other words, you are welcome to use, share and improve this program.
   You are forbidden to forbid anyone else to use, share and improve
   what you give them.   Help stamp out software-hoarding!
-------------------------------------------------------------------------*/

/*

DS89C420 SPECIAL FUNCTION REGISTER LOCATIONS
    
REGISTER    ADDRESS     BIT 7   BIT 6   BIT 5   BIT 4   BIT 3   BIT 2   BIT 1   BIT 0

P0          80h         P0.7    P0.6    P0.5    P0.4    P0.3    P0.2    P0.1    P0.0        Port 0 (P0)
SP          81h                                                                             Stack Pointer (SP)
DPL         82h                                                                             Data Pointer Low 0 (DPL)
DPH         83h                                                                             Data Pointer High 0 (DPH)
DPL1        84h                                                                             Data Pointer Low 1 (DPL1)
DPH1        85h                                                                             Data Pointer High 1 (DPH1)
DPS         86h         ID1     ID0     TSL     AID     -       -       -       SEL         Data Pointer Select (DPS)
PCON        87h         SMOD_0  SMOD0   OFDF    OFDE    GF1     GF0     STOP    IDLE        Power Control (PCON)
TCON        88h         TF1     TR1     TF0     TR0     IE1     IT1     IE0     IT0         Timer/Counter Control (TCON)
TMOD        89h         GATE    C/T     M1      M0      GATE C/ T       M1      M0          Timer Mode Control (TMOD)
TL0         8Ah                                                                             Timer 0 LSB (TL0)
TL1         8Bh                                                                             Timer 1 LSB (TL1)
TH0         8Ch                                                                             Timer 0 MSB (TH0)
TH1         8Dh                                                                             Timer 1 MSB (TH1)
CKCON       8Eh         WD1     WD0     T2M     T1M     T0M     MD2     MD1     MD0         Clock Control (CKCON)
P1          90h         P1.7    P1.6    P1.5    P1.4    P1.3    P1.2    P1.1    P1.0        General Purpose I/O Port 1 (P1)
EXIF        91h         IE5     IE4     IE3     IE2     CKRY    RGMD    RGSL   *BGS         External Interrupt Flag (EXIF)
CKMOD       96h                         T2MH    T1MH    T0MH                                Timer and Serial Port Clock Mode Register (CKMOD)
SCON0       98h        SM0_FE_0 SM1_0   SM2_0   REN_0   TB8_0   RB8_0   TI_0    RI_0        Serial Port 0 Control (SCON0)
SBUF0       99h                                                                             Serial Data Buffer 0 (SBUF0)
ACON        9Dh         PAGEE   PAGES1  PAGES0                                              Address Control (ACON)
P2          A0h         P2.7    P2.6    P2.5    P2.4    P2.3    P2.2    P2.1    P2.0        Port 2 (P2)
IE          A8h         EA      ES1     ET2     ES0     ET1     EX1     ET0     EX0         Interrupt Enable (IE)
SADDR0      A9h                                                                             Slave Address Register 0 (SADDR0)
SADDR1      AAh                                                                             Slave Address Register 1 (SADDR1)
P3          B0h         P3.7    P3.6    P3.5    P3.4    P3.3    P3.2    P3.1    P3.0        Port 3 (P3)
IP1         B1h         -       MPS1    MPT2    MPS0    MPT1    MPX1    MPT0    MPX0        Interrupt Priority 1 (IP1)
IP0         B8h         -       LPS1    LPT2    LPS0    LPT1    LPX1    LPT0    LPX0        Interrupt Priority 0 (IP0)
SADEN0      B9h                                                                             Slave Address Mask Enable Register 0 (SADEN0)
SADEN1      BAh                                                                             Slave Address Mask Enable Register 1 (SADEN1)
SCON1       C0h        SM0_FE_1 SM1_1   SM2_1   REN_1   TB8_1   RB8_1   TI_1    RI_1        Serial Port 1 Control (SCON1)
SBUF1       C1h                                                                             Serial Data Buffer 1 (SBUF1)
ROMSIZE     C2h                                         PRAME  *RMS2   *RMS1   *RMS0        ROM Size Select (ROMSIZE)
BP2         C3h         -       -       LB3     LB2     LB1     MS2     MS1     MS0         ??? not found in DS89C420 Microcontroller User Guide
PMR         C4h         CD1     CD0     SWB     CTM     4X/2X   ALEON   DME1    DME0        Power Management Register (PMR)
STATUS      C5h         PIS2    PIS1    PIS0    -       SPTA1   SPRA1   SPTA0   SPRA0       Status Register (STATUS)
TA          C7h                                                                             Time Access Register (TA)
T2CON       C8h         TF2     EXF2    RCLK    TCLK    EXEN2   TR2     C/ T2   CP/ RL2     Time 2 Control (T2CON)
T2MOD       C9h         -       -       -       -       -       -       T2OE    DCEN        Timer 2 Mode (T2MOD)
RCAP2L      CAh                                                                             Timer 2 Capture LSB (RCAP2L)
RCAP2H      CBh                                                                             Timer 2 Capture LSB (RCAP2H)
TL2         CCh                                                                             Timer 2 LSB (TL2)
TH2         CDh                                                                             Timer 2 MSB (TH2)
PSW         D0h         CY      AC      F0      RS1     RS0     OV      F1      P           Program Status Word (PSW)
FCNTL       D5h         FBUSY   FERR                   *FC3    *FC2    *FC1    *FC0         Flash Memory Control (FCNTL)
FDATA       D6h                                                                             Flash Memory Data (FDATA)
WDCON       D8h         SMOD_1 *POR     EPFI    PFI    *WDIF    WTRF   *EWT    *RWT         Watchdog Control (WDCON)
ACC         E0h                                                                             Accumulator (A or ACC)
EIE         E8h         -       -       -       EWDI    EX5     EX4     EX3     EX2         Extended Interrupt Enable (EIE)
B           F0h                                                                             B Register (B)
EIP1        F1h                                 MPWDI   MPX5    MPX4    MPX3    MPX2        Extended Interrupt Priority 1 (EIP1)
EIP0        F8h         -       -       -       LPWDI   LPX5    LPX4    LPX3    LPX2        Extended Interrupt Priority 0 (EIP0)

                        * = This bit can only be modified with a Timed Access procedure!

*/


#ifndef DS89C420_H
#define DS89C420_H


#define DPL DPL0
#define DPH DPH0
#define IP  IP0
#define ES  ES0
#define RI  RI_0
#define TI  TI_0

// ###############
//  BYTE Register
// ###############

sfr at 0x80 P0      ;    // Port 0
sfr at 0x81 SP      ;    // Stackpointer
sfr at 0x82 DPL0    ;    // Data Pointer 0 Low 
sfr at 0x83 DPH0    ;    // Data Pointer 0 High 
sfr at 0x84 DPL1    ;    // Data Pointer 1 Low 
sfr at 0x85 DPH1    ;    // Data Pointer 1 High 
sfr at 0x86 DPS     ;    // Data Pointer Select
sfr at 0x87 PCON    ;    // Power Control
sfr at 0x88 TCON    ;    // Timer/Counter Control
sfr at 0x89 TMOD    ;    // Timer Mode Control 
sfr at 0x8A TL0     ;    // Timer 0 LSB
sfr at 0x8B TL1     ;    // Timer 1 LSB
sfr at 0x8C TH0     ;    // Timer 0 MSB
sfr at 0x8D TH1     ;    // Timer 1 MSB
sfr at 0x8E CKCON   ;    // Clock Control

sfr at 0x90 P1      ;    // Port 1
sfr at 0x91 EXIF    ;    // External Interupt Flag
sfr at 0x96 CKMOD   ;    // Timer and Serial Port Clock Mode Register
sfr at 0x98 SCON    ;    // Serial Port 0 Control
sfr at 0x99 SBUF    ;    // Serial Data Buffer 0
sfr at 0x9D ACON    ;    // Adress Control

sfr at 0xA0 P2      ;    // Port 2
sfr at 0xA8 IE      ;    // Interupt Enable
sfr at 0xA9 SADDR0  ;    // Slave Address Register 0                                                                                     Slave Address Register 0 (SADDR0)
sfr at 0xAA SADDR1  ;    // Slave Address Register 1         

sfr at 0xB0 P3      ;    // Port 3
sfr at 0xB1 IP1     ;    // Interrupt Priority 1
sfr at 0xB8 IP0     ;    // Interrupt Priority 0
sfr at 0xB9 SADEN0  ;    // Slave Address Mask Enable Register 0
sfr at 0xBA SADEN1  ;    // Slave Address Mask Enable Register 1

sfr at 0xC0 SCON1   ;    // Serial Port 1 Control
sfr at 0xC1 SBUF1   ;    // Serial Data Buffer 1
sfr at 0xC2 ROMSIZE ;    // ROM Size Select
sfr at 0xC4 PMR     ;    // Power Management Register
sfr at 0xC5 STATUS  ;    // Status Register
sfr at 0xC7 TA      ;    // Time Access Register
sfr at 0xC8 T2CON   ;    // Time 2 Control
sfr at 0xC9 T2MOD   ;    // Timer 2 Mode
sfr at 0xCA RCAP2L  ;    // Timer 2 Capture LSB
sfr at 0xCB RCAP2H  ;    // Timer 2 Capture LSB 
sfr at 0xCC TL2     ;    // Timer 2 LSB
sfr at 0xCD TH2     ;    // Timer 2 MSB

sfr at 0xD0 PSW     ;    // Program Status Word
sfr at 0xD5 FCNTL   ;    // Flash Memory Control
sfr at 0xD6 FDATA   ;    // Flash Memory Data
sfr at 0xD8 WDCON   ;    // Watchdog Control

sfr at 0xE0 ACC     ;    // Accumulator
sfr at 0xE8 EIE     ;    // Extended Interrupt Enable
    
sfr at 0xF0 B       ;    // B Register
sfr at 0xF1 EIP1    ;    // Extended Interrupt Priority 1
sfr at 0xF8 EIP0    ;    // Extended Interrupt Priority 0


//  ##########################
//   BIT addressable Register  
//  ##########################


// Port 0 (P0)

sbit at 0x80 P0_0   ;    // PIN 0
sbit at 0x81 P0_1   ;    // PIN 1
sbit at 0x82 P0_2   ;    // PIN 2
sbit at 0x83 P0_3   ;    // PIN 3
sbit at 0x84 P0_4   ;    // PIN 4
sbit at 0x85 P0_5   ;    // PIN 5
sbit at 0x86 P0_6   ;    // PIN 6
sbit at 0x87 P0_7   ;    // PIN 7

// Timer/Counter Control (TCON)

sbit at 0x88 IT0    ;    // Interrupt 0 Type Select
sbit at 0x89 IE0    ;    // Interrupt 0 Edge Detect
sbit at 0x8A IT1    ;    // Interrupt 1 Type Select
sbit at 0x8B IE1    ;    // Interrupt 1 Edge Detect
sbit at 0x8C TR0    ;    // Timer 0 Run Control
sbit at 0x8D TF0    ;    // Timer 0 Overflow Flag
sbit at 0x8E TR1    ;    // Timer 1 Run Control
sbit at 0x8F TF1    ;    // Timer 1 Overflow Flag

// General Purpose I/O Port 1 (P1)

sbit at 0x90 P1_0   ;    // PIN 0
sbit at 0x91 P1_1   ;    // PIN 1
sbit at 0x92 P1_2   ;    // PIN 2
sbit at 0x93 P1_3   ;    // PIN 3
sbit at 0x94 P1_4   ;    // PIN 4
sbit at 0x95 P1_5   ;    // PIN 5
sbit at 0x96 P1_6   ;    // PIN 6
sbit at 0x97 P1_7   ;    // PIN 7

sbit at 0x90 T2     ;    // Timer 2 External Input
sbit at 0x91 T2EX   ;    // Timer 2 Capture/Reload Trigger
sbit at 0x92 RXD1   ;    // Serial Port 1 Receive
sbit at 0x93 TXD1   ;    // Serial Port 1 Transmit
sbit at 0x94 INT2   ;    // External Interrupt 2
sbit at 0x95 INT3   ;    // External Interrupt 3
sbit at 0x96 INT4   ;    // External Interrupt 4
sbit at 0x97 INT5   ;    // External Interrupt 5

// Serial Port 0 Control (SCON0)

sbit at 0x98 RI_0    ;    // Receiver Interrupt Flag
sbit at 0x99 TI_0    ;    // Transmitter Interrupt Flag
sbit at 0x9A RB8_0   ;    // 9th Received Bit State
sbit at 0x9B TB8_0   ;    // 9th Transmission Bit State
sbit at 0x9C REN_0   ;    // Receiver Enable
sbit at 0x9D SM2_0   ;    // Serial Port Mode / Multiple CPU Communications
sbit at 0x9E SM1_0   ;    // Serial Port Mode / No alternate function
sbit at 0x9F SM0_FE_0;    // Serial Port Mode / Framing Error Flag

// Port 2 (P2)

sbit at 0xA0 P2_0   ;    // PIN 0
sbit at 0xA1 P2_1   ;    // PIN 1
sbit at 0xA2 P2_2   ;    // PIN 2
sbit at 0xA3 P2_3   ;    // PIN 3
sbit at 0xA4 P2_4   ;    // PIN 4
sbit at 0xA5 P2_5   ;    // PIN 5
sbit at 0xA6 P2_6   ;    // PIN 6
sbit at 0xA7 P2_7   ;    // PIN 7

// Interrupt Enable (IE)

sbit at 0xA8 EX0    ;    // Enable External Interrupt 0
sbit at 0xA9 ET0    ;    // Enable Timer 0 Interrupt
sbit at 0xAA EX1    ;    // Enable External Interrupt 1
sbit at 0xAB ET1    ;    // Enable Timer 1 Interrupt
sbit at 0xAC ES0    ;    // Enable Serial Port 0 Interrupt
sbit at 0xAD ET2    ;    // Enable Timer 2 Interrupt
sbit at 0xAE ES1    ;    // Enable Serial Port 1 Interrupt
sbit at 0xAF EA     ;    // Global Interrupt Enable

// Purpose I/O Port 3 (P3)

sbit at 0xB0 P3_0   ;    // PIN 0
sbit at 0xB1 P3_1   ;    // PIN 1
sbit at 0xB2 P3_2   ;    // PIN 2
sbit at 0xB3 P3_3   ;    // PIN 3
sbit at 0xB4 P3_4   ;    // PIN 4
sbit at 0xB5 P3_5   ;    // PIN 5
sbit at 0xB6 P3_6   ;    // PIN 6
sbit at 0xB7 P3_7   ;    // PIN 7

sbit at 0xB0 RXD0   ;    // Serial Port 0 Receive
sbit at 0xB1 TXD0   ;    // Serial Port 0 Transmit
sbit at 0xB2 INT0   ;    // External Interrupt 0
sbit at 0xB3 INT1   ;    // External Interrupt 1
sbit at 0xB4 T0     ;    // Counter External Input
sbit at 0xB5 T1     ;    // Timer/Counter External Input
sbit at 0xB6 WR     ;    // External Data Memory Write Strobe
sbit at 0xB7 RD     ;    // External Data Memory Read Strobe

// Interrupt Priority 0 (IP0)

sbit at 0xB8 LPX0   ;    // Least Significant Priority Select Bit for External Interrupt 0
sbit at 0xB9 LPT0   ;    // Least Significant Priority Select Bit for Timer 0 Interrupt
sbit at 0xBA LPX1   ;    // Least Significant Priority Select Bit for External Interrupt 1
sbit at 0xBB LPT1   ;    // Least Significant Priority Select Bit for Timer 1 Interrupt
sbit at 0xBC LPS0   ;    // Least Significant Priority Select Bit for Serial Port 0 Interrupt
sbit at 0xBD LPT2   ;    // Least Significant Priority Select Bit for Timer 2 Interrupt
sbit at 0xBE LPS1   ;    // Least Significant Priority Select Bit for Serial Port 1 Interrupt

// Serial Port 1 Control

sbit at 0xC0 RI_1    ;    // Receiver Interrupt Flag
sbit at 0xC1 TI_1    ;    // Transmitter Interrupt Flag.
sbit at 0xC2 RB8_1   ;    // 9th Received Bit State
sbit at 0xC3 TB8_1   ;    // 9th Transmission Bit State
sbit at 0xC4 REN_1   ;    // Receive Enable
sbit at 0xC5 SM2_1   ;    // Serial Port 1 Mode/Multiple CPU Communications
sbit at 0xC6 SM1_1   ;    // Serial Port 1 Mode/No alternate function
sbit at 0xC7 SM0_FE_1;    // Serial Port 1 Mode/Framing Error Flag

// Time 2 Control (T2CON)

sbit at 0xC8 CP_RL2 ;    // Capture/Reload Select
sbit at 0xC9 C_T2   ;    // Counter/Timer Select
sbit at 0xCA TR2    ;    // Timer 2 Run Control
sbit at 0xCB EXEN2  ;    // Timer 2 External Enable
sbit at 0xCC TCLK   ;    // Transmit Clock Flag
sbit at 0xCD RCLK   ;    // Receive Clock Flag
sbit at 0xCE EXF2   ;    // Timer 2 External Flag
sbit at 0xCF TF2    ;    // Timer 2 Overflow Flag

// Program Status Word (PSW)

sbit at 0xD0 P      ;    // Parity Flag
sbit at 0xD1 F1     ;    // User Flag 1
sbit at 0xD2 OV     ;    // Overflow Flag
sbit at 0xD3 RS0    ;    // Register Bank Select 1
sbit at 0xD4 RS1    ;    // Register Bank Select 1
sbit at 0xD5 F0     ;    // User Flag 0
sbit at 0xD6 AC     ;    // Auxiliary Carry Flag
sbit at 0xD7 CY     ;    // Carry Flag

// Watchdog Control (WDCON)

sbit at 0xD8 RWT    ;    // Reset Watchdog Timer           Timed Access Write Only!
sbit at 0xD9 EWT    ;    // Enable Watchdog Timer Reset    Timed Access Write Only!
sbit at 0xDA WTRF   ;    // Watchdog Timer Reset Flag
sbit at 0xDB WDIF   ;    // Watchdog Interrupt Flag        Timed Access Write Only!
sbit at 0xDC PFI    ;    // Power fail Interrupt Flag
sbit at 0xDD EPFI   ;    // Enable Power fail Interrupt
sbit at 0xDE POR    ;    // Power-on Reset Flag            Timed Access Write Only!
sbit at 0xD8 SMOD_1 ;    // Serial Modification

// Extended Interrupt Enable (EIE)

sbit at 0xE8 EX2    ;    // External Interrupt 2 Enable
sbit at 0xE9 EX3    ;    // External Interrupt 3 Enable
sbit at 0xEA EX4    ;    // External Interrupt 4 Enable
sbit at 0xEB EX5    ;    // External Interrupt 5 Enable
sbit at 0xEC EWDI   ;    // Watchdog Interrupt Enable

// Extended Interrupt Priority 0 (EIP0)

sbit at 0xF8 LPX2   ;    // Most Significant Priority Select Bit for External Interrupt 2
sbit at 0xF9 LPX3   ;    // Most Significant Priority Select Bit for External Interrupt 3
sbit at 0xFA LPX4   ;    // Most Significant Priority Select Bit for External Interrupt 4
sbit at 0xFB LPPX5  ;    // Most Significant Priority Select Bit for External Interrupt 5
sbit at 0xFC LPWDI  ;    // Most Significant Priority Select Bit for the Watchdog Interrupt
 

// ###########################################################
//  BIT definitions for bits that are not directly accessible 
// ###########################################################

// Data Pointer Select (DPS)

#define SEL           0x01    // Data Pointer Select
#define AID           0x10    // Auto Increment/Decrement Enable
#define TSL           0x20    // Toggle Select
#define ID0           0x40    // Increment / Decrement Select for DPTR
#define ID1           0x80    // Increment / Decrement Select for DPTR1

// Power Control (PCON)

#define IDLE          0x01    // Idle Mode Select
#define STOP          0x02    // Stop Mode Select
#define GF0           0x04    // General Purpose User Flag 0
#define GF1           0x08    // General Purpose User Flag 1
#define OFDE          0x10    // Oscillator Fail Detect Enable
#define OFDF          0x20    // Oscillator Fail Detect Flag
#define SMOD0         0x40    // Framing Error Detection Enable
#define SMOD_0        0x80    // Serial Port 0 Baud Rate Doubler Enable

// Timer Mode Control (TMOD)

#define T0_M0         0x01    // Timer 0 Mode Select 0
#define T0_M1         0x02    // Timer 0 Mode Select 1
#define T0_CT         0x04    // Timer 0 Counter/Timer Select
#define T0_GATE       0x08    // Timer 0 Gate Control
#define T1_M0         0x10    // Timer 1 Mode Select 0
#define T1_M1         0x20    // Timer 1 Mode Select 1
#define T1_CT         0x40    // Timer 1 Counter/Timer Select
#define T1_GATE       0x80    // Timer 1 Gate Control

// Clock Control (CKCON)

#define MD0           0x01    // Stretch MOVX Select 0      000 = 0   2 Machine Cycles
#define MD1           0x02    // Stretch MOVX Select 1      001 = 1   3 Machine Cycles (reset default)
#define MD2           0x04    // Stretch MOVX Select 2      111 = 7  12 Machine Cycles
#define T0M           0x08    // Timer 0 Clock Select
#define T1M           0x10    // Timer 1 Clock Select
#define T2M           0x20    // Timer 2 Clock Select
#define WD0           0x40    // Watchdog Timer Mode Select 0
#define WD1           0x80    // Watchdog Timer Mode Select 1

// External Interrupt Flag (EXIF)

#define BGS           0x01    // BGS                     This bit can only be modified with a Timed Access procedure!
#define RGSL          0x02    // Ring Oscillator Select
#define RGMD          0x04    // Ring Mode Status
#define CKRY          0x08    // Clock Ready
#define IE2           0x10    // External Interrupt 2 Flag
#define IE3           0x20    // External Interrupt 3 Flag
#define IE4           0x40    // External Interrupt 4 Flag
#define IE5           0x80    // External Interrupt 5 Flag

// Timer and Serial Port Clock Mode Register (CKMOD)

#define T0MH          0x08    // Timer 0 Clock Mode High Speed Select
#define T1MH          0x10    // Timer 1 Clock Mode High Speed Select
#define T2MH          0x20    // Timer 2 Clock Mode High Speed Select

// Address Control (ACON)

#define PAGES0        0x20    // Page Mode Select 0
#define PAGES1        0x40    // Page Mode Select 1
#define PAGEE         0x80    // Page Mode Enable

// Interrupt Priority 1 (IP1)

#define MPX0          0x01    // Most Significant Priority Select Bit for External Interrupt 0
#define MPT0          0x02    // Most Significant Priority Select Bit for Timer 0 Interrupt
#define MPX1          0x04    // Most Significant Priority Select Bit for External Interrupt 1
#define MPT1          0x08    // Most Significant Priority Select Bit for Timer 1 Interrupt
#define MPS0          0x10    // Most Significant Priority Select Bit for Serial Port 0 Interrupt
#define MPT2          0x20    // Most Significant Priority Select Bit for Timer 2 Interrupt
#define MPS1          0x40    // Most Significant Priority Select Bit for Serial Port 1 Interrupt

// ROM Size Select (ROMSIZE)

#define RMS0          0x01    // ROM Memory Size Select 0   This bit can only be modified with a Timed Access procedure!
#define RMS1          0x02    // ROM Memory Size Select 1   This bit can only be modified with a Timed Access procedure!
#define RMS2          0x04    // ROM Memory Size Select 2   This bit can only be modified with a Timed Access procedure!
#define PRAME         0x08    // Program RAM Enable
#define MOVCX         0x10    // ?????????????????? These bits are reserved. Read data will be 1.
#define TE            0x20    // ?????????????????? These bits are reserved. Read data will be 1.
#define BPF           0x40    // ?????????????????? These bits are reserved. Read data will be 1. 
#define HBPF          0x80    // ?????????????????? These bits are reserved. Read data will be 1.

// Address C3H ????????????????? (BP2)

#define MS0           0x01    // ??????????????????  
#define MS1           0x02    // ??????????????????  not found
#define MS2           0x04    // ??????????????????  in DS89C420
#define LB1           0x08    // ??????????????????  Ultra High Speed
#define LB2           0x10    // ??????????????????  Microcontroller User Guide
#define LB3           0x20    // ??????????????????

// Power Management Register (PMR)

#define DME0          0x01    // Data Memory Enable 0 
#define DME1          0x02    // Data Memory Enable 1         
#define ALEON         0x04    // ALE Enable           
#define _4X_2X        0x08    // Clock Multiplier Selection
#define CTM           0x10    // Crystal Multiplier Enable
#define SWB           0x20    // Switchback Enable
#define CD0           0x40    // Clock Divide Control 0
#define CD1           0x80    // Clock Divide Control 1

// Status Register (STATUS)

#define SPRA0         0x01    // Serial Port 0 Receive Activity Monitor
#define SPTA0         0x02    // Serial Port 0 Transmit Activity Monitor
#define SPRA1         0x04    // Serial Port 1 Receive Activity Monitor
#define SPTA1         0x08    // Serial Port 1 Transmit Activity Monitor
#define PIS0          0x20    // Priority Interrupt Status Bits 0
#define PIS1          0x40    // Priority Interrupt Status Bits 1
#define PIS2          0x80    // Priority Interrupt Status Bits 2

// Timer 2 Mode (T2MOD)

#define DCEN          0x01    // Down Count Enable
#define T2OE          0x02    // Timer 2 Output Enable

// Flash Memory Control (FCNTL)

#define FC0           0x01    // Flash Command Bit 0    This bit can only be modified with a Timed Access procedure!
#define FC1           0x02    // Flash Command Bit 1    This bit can only be modified with a Timed Access procedure!
#define FC2           0x04    // Flash Command Bit 2    This bit can only be modified with a Timed Access procedure!    
#define FC3           0x08    // Flash Command Bit 3    This bit can only be modified with a Timed Access procedure!
#define FERR          0x40    // Flash Error
#define FBUSY         0x80    // Flash Busy

// Extended Interrupt Priority 1 (EIP1)

#define MPX2          0x01    // Most Significant Priority Select Bit for External Interrupt 2
#define MPX3          0x02    // Most Significant Priority Select Bit for External Interrupt 3
#define MPX4          0x04    // Most Significant Priority Select Bit for External Interrupt 4
#define MPX5          0x08    // Most Significant Priority Select Bit for External Interrupt 5
#define MPWDI         0x10    // Most Significant Priority Select Bit for the Watchdog Interrupt

                              
                              // Interrupt numbers: address = (number * 8) + 3 
                            

                              //   Interupt         Vector       Flags         Enalble       Priority Control
                              //------------------------------------------------------------------------------        
                              //                  
#define IE0_VECTOR      0     // Ext. Interrupt 0     03h     IE0 (TCON.1)**  EX0 (IE.0)      LPX0 (IP0.0)
                              //                                                              MPX0 (IP1.0)
                              //
#define TF0_VECTOR      1     // Timer 0 overflow     0Bh     TF0 (TCON.5)*   ET0 (IE.1)      LPT0 (IP0.1)
                              //                                                              MPT0 (IP1.1)
                              //
#define IE1_VECTOR      2     // Ext. Interrupt 1     13h     IE1 (TCON.3)**  EX1 (IE.2)      LPX1 (IP0.2)
                              //                                                              MPX1 (IP1.2)
                              //
#define TF1_VECTOR      3     // Timer 1 overflow     1Bh     TF1 (TCON.7)*   ET1 (IE.3)      LPT1 (IP0.3)
                              //                                                              MPT1 (IP1.3)
                              //
#define SI0_VECTOR      4     // Serial Port 0        23h     RI_0 (SCON0.0)  ES0 (IE.4)      LPS0 (IP0.4)
                              //                              TI_0 (SCON0.1)                  MPS0 (IP1.4)            
                              //      
#define TF2_VECTOR      5     // Timer 2 overflow     2Bh     TF2 (T2CON.7)   ET2 (IE.5)      LPT2 (IP0.5)
                              //                              EXF2 (T2CON.6)                  MPT2 (IP1.5)
                              //
#define PFI_VECTOR      6     // Power Fail           33h     PFI (WDCON.4)   EPFI(WDCON.5)   N/A
                              //
                              //
#define SI1_VECTOR      7     // Serial Port 1        3Bh     RI_1 (SCON1.0)  ES1 (IE.6)      LPS1 (IP0.6)
                              //                              TI_1 (SCON1.1)                  MPS1 (IP1.6)
                              //
#define IE2_Vector      8     // Ext. Interrupt 2     43h     IE2 (EXIF.4)    EX2 (EIE.0)     LPX2 (EIP0.0)
                              //                                                              MPX2 (EIP1.0)
                              //
#define IE3_Vector      9     // Ext. Interrupt 3     4Bh     IE3 (EXIF.5)    EX3 (EIE.1)     LPX3 (EIP0.1)
                              //                                                              MPX3 (EIP1.1)
                              //
#define IE4_Vector      10    // Ext. Interrupt 4     53h     IE4 (EXIF.6)    EX4 (EIE.2)     LPX4 (EIP0.2)
                              //                                                              MPX4 (EIP1.2)
                              //
#define IE5_Vector      11    // Ext. Interrupt 5     5Bh     IE5 (EXIF.7)    EX5 (EIE.3)     LPX5 (EIP0.3)
                              //                                                              MPX5 (EIP1.3)
                              //
#define WDI_Vector      12    // Watchdog             63h     WDIF(WDCON.3)   EWDI(EIE.4)     LPWDI (EIP0.4)
                              //                                                              MPWDI (EIP1.4)
                              //
                              //   * Cleared automatically by hardware when the service routine is vectored to.
                              //
                              //  ** If the interrupt is edge triggered, cleared automatically by hardware 
                              //     when the service routine is vectored to. If the interrupt is level 
                              //     triggered, the flag follows the state of the pin.



#endif





